package pl.kaczanowscy.tomek.testng;

import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;
import org.testng.IResultMap;
import org.testng.ITestContext;
import org.testng.ITestListener;
import org.testng.ITestResult;

/**
 * Generates information on tests results, using Velocity templates.
 * 
 * @author tomek
 * 
 */
public class TestListener implements ITestListener {

	private Map<String, List<String>> failedTests = new HashMap<String, List<String>>();

	private Map<String, List<String>> skippedTests = new HashMap<String, List<String>>();

	public void onFinish(ITestContext testContext) {
		failedTests = generateResultMap(testContext.getFailedTests());
		skippedTests = generateResultMap(testContext.getSkippedTests());
		try {
			printTestsInfo(testContext);
		} catch (IOException e) {
			String msg = "Problem with writing information about tests to files "
					+ e;
			System.out.println(msg);
		}
	}

	private Map<String, List<String>> generateResultMap(IResultMap results) {
		Map<String, List<String>> testsMap = new HashMap<String, List<String>>();
		for (ITestResult result : results.getAllResults()) {
			String testClassName = result.getTestClass().getName();
			if (!testsMap.containsKey(testClassName)) {
				testsMap.put(testClassName, new ArrayList<String>());
			}
			testsMap.get(testClassName).add(result.getName());
		}
		return testsMap;
	}

	private void printTestsInfo(ITestContext testContext) throws IOException {
		int passedNb = testContext.getPassedTests().size();
		int failedNb = testContext.getFailedTests().size();
		int skippedNb = testContext.getSkippedTests().size();
		VelocityContext context = new VelocityContext();
		context.put("totalNb", passedNb + skippedNb + failedNb);
		context.put("passedNb", passedNb);
		context.put("failedNb", failedNb);
		context.put("skippedNb", skippedNb);
		context.put("failedTests", failedTests);
		context.put("skippedTests", skippedTests);
		VelocityEngine ve = new VelocityEngine();
		ve.setProperty("file.resource.loader.class",
				ClasspathResourceLoader.class.getName());

		try {
			ve.init();
			writeFile(ve, context, "templates/summary_txt.vm",
					"test_summary.txt");
			writeFile(ve, context, "templates/summary_html.vm",
					"test_summary.html");
		} catch (Exception e) {
			// do something more interesting here
			e.printStackTrace();
		}
	}

	private void writeFile(VelocityEngine ve, VelocityContext context,
			String vmPath, String fileName) throws ResourceNotFoundException,
			ParseErrorException, Exception {
		Template t = ve.getTemplate(vmPath);
		StringWriter writer = new StringWriter();
		t.merge(context, writer);
		FileWriter fileWriter = new FileWriter(fileName, true);
		fileWriter.append(writer.toString());
		fileWriter.flush();
		fileWriter.close();
	}

	public void onStart(ITestContext arg0) {
		// intentionally left blank
	}

	public void onTestFailedButWithinSuccessPercentage(ITestResult arg0) {
		// intentionally left blank
	}

	public void onTestFailure(ITestResult result) {
		// intentionally left blank
	}

	public void onTestSkipped(ITestResult result) {
		// intentionally left blank
	}

	public void onTestStart(ITestResult result) {
		// intentionally left blank
	}

	public void onTestSuccess(ITestResult result) {
		// intentionally left blank
	}
}
